<?php namespace RVR\Controller;

use DateTime;
use RVR\PersistentData\Model\Community;
use RVR\PersistentData\Model\User;
use RVR\Repository\CommunityMemberRepository;
use RVR\Repository\EventRepository;
use SokoWeb\Interfaces\Authentication\IAuthenticationRequired;
use SokoWeb\Interfaces\Response\IContent;
use SokoWeb\Response\HtmlContent;

class HomeController implements IAuthenticationRequired
{
    private CommunityMemberRepository $communityMemberRepository;

    private EventRepository $eventRepository;

    public function __construct()
    {
        $this->communityMemberRepository = new CommunityMemberRepository();
        $this->eventRepository = new EventRepository();
    }

    public function isAuthenticationRequired(): bool
    {
        return true;
    }

    public function getHome(): IContent
    {
        /**
        * @var User $user
        */
        $user = \Container::$request->user();

        $ownCommunityMembers = $this->communityMemberRepository->getAllByUser($user, true, ['community']);
        $communities = [];
        foreach ($ownCommunityMembers as $ownCommunityMember) {
            $communities[] = $ownCommunityMember->getCommunity();
        }
        usort($communities, function($a, $b) {
            return strnatcmp($a->getName(), $b->getName());
        });

        return new HtmlContent('home', [
            'communities' => $communities,
            'upcomingAndRecentEvents' => iterator_to_array($this->eventRepository->getUpcomingAndRecentByUser($user, new DateTime(), 30, 3, true, ['community']))
        ]);
    }
}
